@@ -24,6 +24,12 @@ class TopologicalNetwork(ProbeableObj):
                                                               self.adjacency.flatten()*1))
         self.n_nodes = len(self.adjacency)
     
+    def text_plot(self):
+        for i in range(len(self.adjacency)):
+            for j in range(len(self.adjacency)): 
+                if self.adjacency[i][j]==True:
+                    print str(j)+"--->"+str(i)     
+
     def indegree(self):
         return self.adjacency.sum(axis=0)
     
@@ -44,6 +50,78 @@ class TopologicalNetwork(ProbeableObj):
         node_radius = 10
         
         drawing.create_text(200,10,text = "Network:"+str(id(self)))
+
+        list_of_coordinates = [(radius*math.sin(2*math.pi*n/n_nodes)+canvas_size/2,radius*math.cos(2*math.pi*n/n_nodes)+canvas_size/2) for n in range(n_nodes)]
+        
+        for linksto,node in enumerate(self.adjacency):
+            for linksfrom,link in enumerate(node):
+                
+                if linksto == linksfrom and link==True:
+                    angle = math.atan2(list_of_coordinates[linksto][1]-200,
+                                      list_of_coordinates[linksto][0]-200)
+                    
+                    drawing.create_line(list_of_coordinates[linksto][0]+node_radius*math.cos(angle),
+                                        list_of_coordinates[linksto][1]+node_radius*math.sin(angle),
+                                        list_of_coordinates[linksto][0]+node_radius*2*(math.cos(angle+20)),
+                                        list_of_coordinates[linksto][1]+node_radius*2*math.sin(angle+20),
+                                        list_of_coordinates[linksto][0]+node_radius*4*(math.cos(angle)),
+                                        list_of_coordinates[linksto][1]+node_radius*4*math.sin(angle),
+                                        list_of_coordinates[linksto][0]+node_radius*2*math.cos(angle-20),
+                                        list_of_coordinates[linksto][1]+node_radius*2*(math.sin(angle-20)),
+                                        list_of_coordinates[linksto][0]+node_radius*math.cos(angle),
+                                        list_of_coordinates[linksto][1]+node_radius*math.sin(angle),
+                                        smooth=True,joinstyle="round",fill="black",width=2,arrow="last"
+                                        )
+                
+                elif link == True: 
+                    angle = math.atan2(list_of_coordinates[linksto][1]-list_of_coordinates[linksfrom][1],
+                                   list_of_coordinates[linksto][0]-list_of_coordinates[linksfrom][0])
+                
+                    drawing.create_line(list_of_coordinates[linksfrom][0]+node_radius*math.cos(angle),
+                                        list_of_coordinates[linksfrom][1]+node_radius*math.sin(angle),
+                                        list_of_coordinates[linksto][0]-node_radius*math.cos(angle),
+                                        list_of_coordinates[linksto][1]-node_radius*math.sin(angle),
+                                        fill="black",width=2,arrow="last")
+        
+        for node_ctr,(x,y) in enumerate(list_of_coordinates):
+
+            if type(self) != Network:
+                node_color = "white"
+                text_color = "black"
+            elif self.state == num.array([[]]):
+                node_color = "white"
+                text_color = "black"
+            else:
+                if self.state[-1][node_ctr] == True: 
+                    node_color = "black"
+                    text_color = "white"
+                else:                
+                    node_color = "white"
+                    text_color = "black"
+                    
+            drawing.create_oval(x-node_radius,y-node_radius,x+node_radius,y+node_radius,width=2,fill=node_color)
+            drawing.create_text(x,y,text =  str(node_ctr),fill = text_color, font="Arial")
+            
+
+        
+        drawing.pack()
+        window.mainloop()
+        
+    def save_plot(self,filename):
+        """Opens a window, draws the graph into the window.
+           Requires Tk, and of course a windowing system.
+        """
+        import Tkinter as tk
+        import math
+        window= tk.Tk()
+        canvas_size = 400
+        drawing = tk.Canvas(window, height=canvas_size, width=canvas_size, background="white")
+        n_nodes = self.n_nodes
+        radius = 150
+        node_radius = 10
+        
+        drawing.create_text(200,10,text = "Network:"+str(id(self)))
+        drawing.pack()
         
         list_of_coordinates = [(radius*math.sin(2*math.pi*n/n_nodes)+canvas_size/2,radius*math.cos(2*math.pi*n/n_nodes)+canvas_size/2) for n in range(n_nodes)]
         
@@ -78,7 +156,11 @@ class TopologicalNetwork(ProbeableObj):
                                         fill="black",width=2,arrow="last")
         
         for node_ctr,(x,y) in enumerate(list_of_coordinates):
-            if self.state == []:
+
+            if type(self) != Network:
+                node_color = "white"
+                text_color = "black"
+            elif self.state == num.array([[]]):
                 node_color = "white"
                 text_color = "black"
             else:
@@ -93,12 +175,72 @@ class TopologicalNetwork(ProbeableObj):
             drawing.create_text(x,y,text =  str(node_ctr),fill = text_color, font="Arial")
             
 
-        
+        drawing.update()
         drawing.pack()
-        window.mainloop()
+        drawing.postscript(file=filename+".ps")
+        
+        #window.mainloop
 
+    def laplacian(self):  
+        """
+        Returns the graph laplacian of the network
+        """  
+        symmetric = self.adjacency+self.adjacency.T-num.diag(self.adjacency.diagonal())
+        degrees = num.diag(symmetric.sum(axis=0))
+        laplacian = degrees-symmetric
+        return laplacian
         
+    def directed_laplacian(self):
+        """
+        Returns the laplacian of the network. It differs from laplacian function by using
+        the original adjacency matrix, not the symmetricised version of it. 
+        """
+        original = self.adjacency-num.diag(self.adjacency.diagonal())
+        degrees = num.diag(original.sum(axis=0)+original.sum(axis=1))
+        laplacian = degrees-original
+        return laplacian
         
+    def indegree_laplacian(self):
+        """
+        Returns the laplacian composed of in-degrees of the nodes
+        """
+        original = self.adjacency-num.diag(self.adjacency.diagonal())
+        degrees = num.diag(original.sum(axis=1))
+        laplacian = degrees-original
+        return laplacian  
+    
+    def outdegree_laplacian(self):
+        """
+        Returns the laplacian composed of out-degrees of the nodes
+        """
+        original = self.adjacency-num.diag(self.adjacency.diagonal())
+        degrees = num.diag(original.sum(axis=0))
+        laplacian = degrees-original
+        return laplacian  
+      
+    def laplacian_eigvals(self):
+        """
+        Returns an ordered array of eigenvalues of the laplacian.
+        """
+        ordered_list = []
+        values = []
+        eigenvalues = num.linalg.eigvals(self.laplacian())   
+        for i in range(len(self.adjacency)):
+            values.append(eigenvalues[i])
+        for i in range(len(self.adjacency)):
+            ordered_list.append(min(values))
+            values.remove(min(values))
+        return num.array(ordered_list)
+        
+    def spectral_distance(self,other):
+        """
+        Computes spectral distance between networks.
+        """
+        difference = self.laplacian_eigvals()-other.laplacian_eigvals()
+        distance = difference * difference
+        spec_distance = distance.sum()
+        return spec_distance
+
     def is_connected(self):
         """
         Returns True if the graph is connected, False if not.
@@ -114,6 +256,14 @@ class TopologicalNetwork(ProbeableObj):
         determinant = num.linalg.det(laplacian +num.ones((len(laplacian),len(laplacian) )))
         return not(determinant == 0)
     
+    def remove_self_connection(self):
+        """
+        Removes self connections of the nodes in the network.
+        """
+        diagonal = num.diag(num.diag(self.adjacency))
+        new_adjacency = self.adjacency - diagonal
+        self.adjacency = new_adjacency
+            
     def copy(self):
         """
         Returns a copy of the Topological Network object. 
@@ -172,7 +322,6 @@ class Network(TopologicalNetwork,Element):
         state_vec  
     '''
     def __init__ (self,adjacency_matrix,mask,function,state_vec=None):
-
         Element.__init__(self)
         TopologicalNetwork.__init__(self,adjacency_matrix)
         self.n_nodes= num.size(adjacency_matrix,0)
@@ -181,7 +330,6 @@ class Network(TopologicalNetwork,Element):
             state_vec= (num.random.random((1,self.n_nodes))< 0.5)
         self.state=num.array(state_vec)
         self.function = function
-        self.equilibria = num.array([None]*(2**self.n_nodes))
     
     def __str__(self):
         return str(id(self))
@@ -367,12 +515,10 @@ class Network(TopologicalNetwork,Element):
             self.orbits = num.array([None]*2**self.n_nodes)
         
         binspace = range(0,num.power(2,self.n_nodes))
-
+  
         for state in binspace:
             self.search_equilibrium(2**self.n_nodes,state,orbit_extraction)  
-
         self.populate_probes(probes.populate_equilibria)
-        return self
-    
+
     
 
